import logging

from peek_plugin_diagram._private.server.api.DiagramImportApi import DiagramImportApi
from peek_plugin_diagram._private.server.api.DiagramViewerApi import DiagramViewerApi
from peek_plugin_diagram._private.server.controller.BranchUpdateController import \
    BranchUpdateController
from peek_plugin_diagram._private.server.controller.DispImportController import \
    DispImportController
from peek_plugin_diagram._private.server.controller.LookupImportController import \
    LookupImportController
from peek_plugin_diagram._private.server.controller.StatusController import \
    StatusController
from peek_plugin_diagram.server.DiagramApiABC import DiagramApiABC
from peek_plugin_diagram.server.DiagramImportApiABC import DiagramImportApiABC
from peek_plugin_diagram.server.DiagramViewerApiABC import DiagramViewerApiABC

logger = logging.getLogger(__name__)


class DiagramApi(DiagramApiABC):
    def __init__(self, mainController: StatusController,
                 dispImportController: DispImportController,
                 lookupImportController: LookupImportController,
                 branchUpdateController: BranchUpdateController,
                 ormSessionCreator):
        self._viewerApi = DiagramViewerApi(ormSessionCreator)
        self._importApi = DiagramImportApi(
            mainController, dispImportController,
            lookupImportController, branchUpdateController
        )

    def shutdown(self):
        self._importApi.shutdown()
        self._viewerApi.shutdown()

        self._importApi = None
        self._viewerApi = None

    def importApi(self) -> DiagramImportApiABC:
        return self._importApi

    def viewerApi(self) -> DiagramViewerApiABC:
        return self._viewerApi



import logging
from typing import Dict

from peek_abstract_chunked_index.private.client.handlers.ACICacheHandlerABC import \
    ACICacheHandlerABC
from peek_abstract_chunked_index.private.tuples.ACIUpdateDateTupleABC import \
    ACIUpdateDateTupleABC
from peek_plugin_diagram._private.PluginNames import diagramFilt
from peek_plugin_diagram._private.tuples.location_index.LocationIndexUpdateDateTuple import \
    LocationIndexUpdateDateTuple

logger = logging.getLogger(__name__)

clientLocationIndexWatchUpdateFromDeviceFilt = {
    'key': "clientLocationIndexWatchUpdateFromDevice"
}
clientLocationIndexWatchUpdateFromDeviceFilt.update(diagramFilt)


# ModelSet HANDLER
class LocationIndexCacheHandler(ACICacheHandlerABC):
    _UpdateDateTuple: ACIUpdateDateTupleABC = LocationIndexUpdateDateTuple
    _updateFromServerFilt: Dict = clientLocationIndexWatchUpdateFromDeviceFilt
    _logger: logging.Logger = logger




import logging

from sqlalchemy import Column, Index, BigInteger
from sqlalchemy import ForeignKey
from sqlalchemy import Integer, String
from vortex.Tuple import Tuple, addTupleType

from peek_abstract_chunked_index.private.tuples.ACIProcessorQueueTupleABC import \
    ACIProcessorQueueTupleABC
from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix
from peek_plugin_diagram._private.storage.DeclarativeBase import DeclarativeBase

logger = logging.getLogger(__name__)


@addTupleType
class BranchIndexCompilerQueue(Tuple, DeclarativeBase,
                               ACIProcessorQueueTupleABC):
    __tablename__ = 'BranchIndexCompilerQueue'
    __tupleType__ = diagramTuplePrefix + 'BranchIndexCompilerQueueTable'

    id = Column(BigInteger, primary_key=True, autoincrement=True)

    modelSetId = Column(Integer,
                        ForeignKey('ModelSet.id', ondelete='CASCADE'),
                        nullable=False,
                        autoincrement=True)

    chunkKey = Column(String, primary_key=True)

    __table_args__ = (
        Index("idx_BICompQueue_modelSetId_chunkKey", modelSetId, chunkKey, unique=False),
    )

    @classmethod
    def sqlCoreLoad(cls, row):
        return BranchIndexCompilerQueue(id=row.id,
                                        modelSetId=row.modelSetId,
                                        chunkKey=row.chunkKey)

    @property
    def ckiUniqueKey(self):
        return self.chunkKey



from typing import List

from copy import copy

from peek_plugin_diagram._private.PluginNames import diagramFilt
from peek_plugin_diagram._private.storage.Display import DispLevel, DispTextStyle, \
    DispLayer, DispColor, DispLineStyle
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TupleDataObservableHandler
from vortex.handler.TupleDataObserverClient import TupleDataObserverClient

lookupCachePayloadFilt = dict(key="client.lookup.update")
lookupCachePayloadFilt.update(diagramFilt)


class LookupCacheController:
    """ Lookup Cache Controller

    This class caches the lookups in each client.

    """
    #: This stores the cache of grid data for the clients
    _levelLookups: List[DispLevel] = None
    _layerLookups: List[DispLayer] = None
    _colorLookups: List[DispColor] = None
    _lineStyleLookups: List[DispLineStyle] = None
    _textStyleLookups: List[DispTextStyle] = None

    def __init__(self, tupleObserver: TupleDataObserverClient):
        self._tupleObserver = tupleObserver
        self._tupleObservable = None

    def setTupleObservable(self, tupleObservable: TupleDataObservableHandler):
        self._tupleObservable = tupleObservable

    def start(self):
        (self._tupleObserver
         .subscribeToTupleSelector(TupleSelector(DispLevel.tupleType(), {}))
         .subscribe(self._processNewTuples))

        (self._tupleObserver
         .subscribeToTupleSelector(TupleSelector(DispLayer.tupleType(), {}))
         .subscribe(self._processNewTuples))

        (self._tupleObserver
         .subscribeToTupleSelector(TupleSelector(DispColor.tupleType(), {}))
         .subscribe(self._processNewTuples))

        (self._tupleObserver
         .subscribeToTupleSelector(TupleSelector(DispLineStyle.tupleType(), {}))
         .subscribe(self._processNewTuples))

        (self._tupleObserver
         .subscribeToTupleSelector(TupleSelector(DispTextStyle.tupleType(), {}))
         .subscribe(self._processNewTuples))

    def shutdown(self):
        self._tupleObservable = None
        self._tupleObserver = None

        self._levelLookups = []
        self._layerLookups = []
        self._colorLookups = []
        self._lineStyleLookups = []
        self._textStyleLookups = []

    def _processNewTuples(self, lookupTuples):

        if not lookupTuples:
            return

        firstTupleType = lookupTuples[0].tupleType()
        if DispLevel.tupleType() == firstTupleType:
            self._levelLookups = lookupTuples

        elif DispLayer.tupleType() == firstTupleType:
            self._layerLookups = lookupTuples

        elif DispColor.tupleType() == firstTupleType:
            self._colorLookups = lookupTuples

        elif DispLineStyle.tupleType() == firstTupleType:
            self._lineStyleLookups = lookupTuples

        elif DispTextStyle.tupleType() == firstTupleType:
            self._textStyleLookups = lookupTuples

        else:
            raise NotImplementedError(
                "Cache not implemented for %s" % firstTupleType)

        self._tupleObservable.notifyOfTupleUpdate(
            TupleSelector(firstTupleType, {})
        )

    def lookups(self, lookupTupleType) -> List:
        if DispLevel.tupleType() == lookupTupleType:
            return copy(self._levelLookups)

        if DispLayer.tupleType() == lookupTupleType:
            return copy(self._layerLookups)

        if DispColor.tupleType() == lookupTupleType:
            return copy(self._colorLookups)

        if DispLineStyle.tupleType() == lookupTupleType:
            return copy(self._lineStyleLookups)

        if DispTextStyle.tupleType() == lookupTupleType:
            return copy(self._textStyleLookups)

        raise NotImplementedError("Cache not implemented for %s" % lookupTupleType)



from datetime import datetime

from vortex.Tuple import addTupleType, TupleField, Tuple

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix


@addTupleType
class EncodedGridTuple(Tuple):
    """ Encoded Grid Tuple

    This tuple stores a pre-encoded version of a GridTuple

    """
    __tupleType__ = diagramTuplePrefix + "EncodedGridTuple"

    gridKey: str = TupleField()

    # A GridTuple, already encoded and ready for storage in the clients cache
    encodedGridTuple: str = TupleField()

    lastUpdate: datetime = TupleField()

    @property
    def ckiChunkKey(self):
        return self.gridKey

    @property
    def ckiHasEncodedData(self) -> bool:
        return bool(self.encodedGridTuple)

    @property
    def ckiLastUpdate(self):
        return self.lastUpdate



"""Increased size of importGroupHash

Peek Plugin Database Migration Script

Revision ID: e9d80c4c087d
Revises: 4b943b584307
Create Date: 2017-10-25 10:03:26.694547

"""

# revision identifiers, used by Alembic.
revision = 'e9d80c4c087d'
down_revision = '4b943b584307'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    op.alter_column('DispBase', 'importGroupHash', type_=sa.String(length=200), nullable=True, schema='pl_diagram')


def downgrade():
    op.alter_column('DispBase', 'importGroupHash', type_=sa.String(length=100), nullable=True, schema='pl_diagram')



import logging
from typing import List

from twisted.internet.defer import Deferred, inlineCallbacks
from vortex.DeferUtil import deferToThreadWrapWithLogger

from peek_plugin_base.storage.StorageUtil import makeOrmValuesSubqueryCondition
from peek_plugin_diagram._private.storage.GridKeyIndex import GridKeyIndex
from peek_plugin_diagram._private.storage.LiveDbDispLink import LiveDbDispLink
from peek_plugin_livedb.server.LiveDBReadApiABC import LiveDBReadApiABC
from peek_plugin_livedb.server.LiveDBWriteApiABC import LiveDBWriteApiABC

logger = logging.getLogger(__name__)


class LiveDbWatchController:
    """ Watch Grid Controller

    This controller handles most of the interactions with the LiveDB plugin..

    That is :

    1) Informing the LiveDB that the keys are being watched

    2) Sending updates for these watched grids to the clients (???? is this the right spot?)

    3) Computing display values for the live db

    4) Queueing display updates to be compiled, based on events from the livedb.

    """

    def __init__(self, liveDbWriteApi: LiveDBWriteApiABC,
                 liveDbReadApi: LiveDBReadApiABC,
                 dbSessionCreator):
        self._liveDbWriteApi = liveDbWriteApi
        self._liveDbReadApi = liveDbReadApi
        self._dbSessionCreator = dbSessionCreator

    def shutdown(self):
        pass

    @inlineCallbacks
    def updateClientWatchedGrids(self, clientId: str, gridKeys: List[str]) -> Deferred:
        """ Update Client Watched Grids

        Tell the server that these grids are currently being watched by users.

        :param clientId: A unique identifier of the client (Maybe it's vortex uuid)
        :param gridKeys: A list of grid keys that this client is observing.
        :returns: Nothing
        """

        try:
            liveDbKeys = yield self.getLiveDbKeys(gridKeys)
            self._liveDbWriteApi.prioritiseLiveDbValueAcquisition(
                'pofDiagram', liveDbKeys
            )

        except Exception as e:
            logger.exception(e)

    @deferToThreadWrapWithLogger(logger)
    def getLiveDbKeys(self, gridKeys) -> List[str]:

        session = self._dbSessionCreator()
        try:
            return [t[0] for t in
                    session.query(LiveDbDispLink.liveDbKey)
                        .join(GridKeyIndex,
                              GridKeyIndex.dispId == LiveDbDispLink.dispId)
                        .filter(makeOrmValuesSubqueryCondition(
                        session, GridKeyIndex.gridKey, gridKeys
                    ))
                        .yield_per(1000)
                        .distinct()]
        finally:
            session.close()



"""Added DispNull

Peek Plugin Database Migration Script

Revision ID: 6d3a96cf1955
Revises: aa76b059b81e
Create Date: 2019-05-29 10:51:00.872234

"""

# revision identifiers, used by Alembic.
revision = '6d3a96cf1955'
down_revision = 'aa76b059b81e'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('DispNull',
    sa.Column('id', sa.Integer(), autoincrement=False, nullable=False),
    sa.Column('geomJson', sa.String(), nullable=False),
    sa.ForeignKeyConstraint(['id'], ['pl_diagram.DispBase.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )




def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('DispNull', schema='pl_diagram')
    # ### end Alembic commands ###


import json
import logging
from typing import List, Optional, Any, Dict

from twisted.internet.defer import inlineCallbacks, returnValue, DeferredSemaphore

from peek_plugin_diagram._private.storage.Display import DispColor, DispLayer, DispLevel, \
    DispLineStyle, DispTextStyle
from peek_plugin_diagram._private.storage.ModelSet import getOrCreateModelSet, \
    getOrCreateCoordSet
from peek_plugin_diagram.tuples.lookups.ImportDispColorTuple import ImportDispColorTuple
from peek_plugin_diagram.tuples.lookups.ImportDispLayerTuple import ImportDispLayerTuple
from peek_plugin_diagram.tuples.lookups.ImportDispLevelTuple import ImportDispLevelTuple
from peek_plugin_diagram.tuples.lookups.ImportDispLineStyleTuple import \
    ImportDispLineStyleTuple
from peek_plugin_diagram.tuples.lookups.ImportDispTextStyleTuple import \
    ImportDispTextStyleTuple
from vortex.DeferUtil import deferToThreadWrapWithLogger
from vortex.Tuple import TUPLE_TYPES_BY_NAME

logger = logging.getLogger(__name__)

ORM_TUPLE_MAP: Dict[str, Any] = {
    ImportDispColorTuple.tupleType(): DispColor,
    ImportDispLayerTuple.tupleType(): DispLayer,
    ImportDispLevelTuple.tupleType(): DispLevel,
    ImportDispLineStyleTuple.tupleType(): DispLineStyle,
    ImportDispTextStyleTuple.tupleType(): DispTextStyle,
}


class LookupImportController:
    def __init__(self, dbSessionCreator):
        self._dbSessionCreator = dbSessionCreator

        self._semaphore = DeferredSemaphore(1)

    def shutdown(self):
        pass

    @inlineCallbacks
    def importLookups(self, modelSetKey: str, coordSetKey: Optional[str],
                      lookupTupleType: str, lookupTuples: List,
                      deleteOthers: bool, updateExisting: bool):

        yield self._semaphore.run(self._importInThread, modelSetKey, coordSetKey,
                                   lookupTupleType, lookupTuples,
                                   deleteOthers, updateExisting)

        logger.debug("TODO, Notify the observable")

        return True

    @deferToThreadWrapWithLogger(logger)
    def _importInThread(self, modelSetKey: str, coordSetKey: str, tupleType: str,
                        tuples,
                        deleteOthers: bool, updateExisting: bool):
        LookupType = ORM_TUPLE_MAP[tupleType]

        if LookupType == DispLineStyle:
            self._convertLineStyles(tuples)

        itemsByImportHash = {}

        addCount = 0
        updateCount = 0
        deleteCount = 0

        ormSession = self._dbSessionCreator()
        try:

            modelSet = getOrCreateModelSet(ormSession, modelSetKey)
            coordSet = None

            if coordSetKey:
                coordSet = getOrCreateCoordSet(
                    ormSession, modelSetKey, coordSetKey)

                all = (ormSession.query(LookupType)
                       .filter(LookupType.coordSetId == coordSet.id)
                       .all())

            else:
                all = (ormSession.query(LookupType)
                       .filter(LookupType.modelSetId == modelSet.id)
                       .all())

            def updateFks(lookup):
                if hasattr(lookup, "coordSetId"):
                    assert coordSet
                    lookup.coordSetId = coordSet.id
                else:
                    lookup.modelSetId = modelSet.id

            for lookup in all:
                # Initialise
                itemsByImportHash[lookup.importHash] = lookup

            for lookup in tuples:
                importHash = str(lookup.importHash)

                # If it's an existing item, update it
                if importHash in itemsByImportHash:
                    existing = itemsByImportHash.pop(importHash)

                    if updateExisting:
                        for fieldName in lookup.tupleFieldNames():
                            setattr(existing, fieldName,
                                    getattr(lookup, fieldName))

                        updateFks(existing)
                        updateCount += 1

                # If it's a new item, create it
                else:
                    newTuple = LookupType()

                    for fieldName in lookup.tupleFieldNames():
                        if fieldName in ("id", "coordSetId", "modelSetId"):
                            continue
                        setattr(newTuple, fieldName,
                                getattr(lookup, fieldName))

                    updateFks(newTuple)
                    ormSession.add(newTuple)
                    addCount += 1

            if deleteOthers:
                for lookup in list(itemsByImportHash.values()):
                    ormSession.delete(lookup)
                    deleteCount += 1

            try:
                ormSession.commit()

            except Exception as e:
                ormSession.rollback()
                logger.exception(e)
                raise

            logger.debug("Updates for %s received, Added %s, Updated %s, Deleted %s",
                        tupleType, addCount, updateCount, deleteCount)

        except Exception as e:
            logger.exception(e)
            raise

        finally:
            ormSession.close()

    @deferToThreadWrapWithLogger(logger)
    def getLookups(self, modelSetKey: str, coordSetKey: Optional[str],
                   tupleType: str):

        LookupType = ORM_TUPLE_MAP[tupleType]

        ormSession = self._dbSessionCreator()
        try:

            modelSet = getOrCreateModelSet(ormSession, modelSetKey)

            if coordSetKey:
                coordSet = getOrCreateCoordSet(
                    ormSession, modelSetKey, coordSetKey)

                all = (ormSession.query(LookupType)
                       .filter(LookupType.coordSetId == coordSet.id)
                       .all())

            else:
                all = (ormSession.query(LookupType)
                       .filter(LookupType.modelSetId == modelSet.id)
                       .all())

            importTuples = []
            ImportTuple = TUPLE_TYPES_BY_NAME[tupleType]

            for ormTuple in all:
                newTuple = ImportTuple()

                for fieldName in newTuple.tupleFieldNames():
                    if fieldName == 'modelSetKey':
                        newTuple.modelSetKey = modelSetKey

                    elif fieldName == 'coordSetKey':
                        newTuple.coordSetKey = coordSetKey

                    else:
                        setattr(newTuple, fieldName,
                                getattr(ormTuple, fieldName))

                importTuples.append(newTuple)

            return importTuples

        except Exception as e:
            logger.exception(e)

        finally:
            ormSession.close()

    def _convertLineStyles(self, importLineStyles: List[ImportDispTextStyleTuple]):
        for style in importLineStyles:
            dp = style.dashPattern

            if dp is None:
                continue

            if not isinstance(dp, list):
                dp = [dp]

            style.dashPattern = json.dumps(dp)



import logging
import typing
from datetime import datetime
from typing import List

import pytz
import json
from peek_plugin_base.worker import CeleryDbConn
from peek_plugin_diagram._private.storage.Display import DispBase
from peek_plugin_diagram._private.storage.GridKeyIndex import GridKeyCompilerQueue, \
    GridKeyIndex
from peek_plugin_diagram._private.tuples.branch.BranchTuple import \
    BranchTuple
from peek_plugin_diagram._private.worker.tasks.DispCompilerTask import _packDispJson
from sqlalchemy import select
from vortex.Tuple import Tuple

logger = logging.getLogger(__name__)


def _deleteBranchDisps(conn, branchIds: List[int]) -> None:
    """ Queue Grids for Removed Branch Disps

    This method queues grids for compile that contain disps that are removed.

    NOTE: The branches will be removed by a cascading constraint, we only have
    to queue the grids for recompile.

    """
    if not branchIds:
        return

    for branchId in branchIds:
        if branchId is None:
            raise Exception("BranchID is None, it shouldn't be")

    dispBaseTable = DispBase.__table__
    gridKeyIndexTable = GridKeyIndex.__table__

    queueTable = GridKeyCompilerQueue.__table__

    results = conn.execute(
        select(distinct=True,
               columns=[gridKeyIndexTable.c.gridKey, gridKeyIndexTable.c.coordSetId],
               whereclause=dispBaseTable.c.branchId.in_(branchIds))
            .select_from(gridKeyIndexTable.join(dispBaseTable))
    ).fetchall()

    if results:
        conn.execute(
            queueTable.insert(),
            [dict(coordSetId=item.coordSetId, gridKey=item.gridKey) for item in results]
        )

    # Delete existing Disps
    logger.info("Deleting disps with branchId %s" % branchIds)
    conn.execute(
        dispBaseTable.delete(dispBaseTable.c.branchId.in_(branchIds))
    )


def _convertBranchDisps(newBranches: List[BranchTuple]) -> typing.Tuple[List, List]:
    """ Insert Disps for Branch

    1) Insert new Disps
    2) Queue disps for recompile

    """
    startTime = datetime.now(pytz.utc)
    # Create state arrays
    newDisps = []
    dispIdsToCompile = []

    # Convert the branch disps into database disps
    for newBranch in newBranches:

        branchDisps = _convertJsonDispsToTuples(newBranch)

        if not branchDisps:
            continue

        # Create the map from the UI temp ID to the DB ID
        oldDispIdMap = {}

        # Set the IDs of the new Disps
        newIdGen = CeleryDbConn.prefetchDeclarativeIds(DispBase, len(branchDisps))
        for disp in branchDisps:
            oldDispId = disp.id
            disp.id = next(newIdGen)
            oldDispIdMap[oldDispId] = disp.id
            dispIdsToCompile.append(disp.id)

            newDisps.append(disp)

        # Update the group IDs
        for disp in branchDisps:
            if disp.groupId in oldDispIdMap:
                disp.groupId = oldDispIdMap[disp.groupId]

        # Recreate the branch disp json as per the structure from the DispBase tables
        # Just to be clear, this is converting it one way and then converting it back.
        # It ensures the data is consistent. (Which it should be if all was right)
        # It also sets the "hashId"

        # Create the map from the UI temp ID to the DB ID
        oldDispHashIdMap = {}
        newBranchDispItems = []

        newBranch.disps = []
        for disp in branchDisps:
            oldDispHashId = disp.hashId
            # This assigns the hashId to the jsonDict and disp
            newJsonDict = _packDispJson(disp, disp.tupleToSmallJsonDict())
            newBranch.disps.append(newJsonDict)

            oldDispHashIdMap[oldDispHashId] = disp.hashId
            newBranchDispItems.append((disp, newJsonDict))

        for disp, jsonDict in newBranchDispItems:
            if disp.replacesHashId in oldDispHashIdMap:
                disp.replacesHashId = oldDispHashIdMap.get(disp.replacesHashId)
                jsonDict['rid'] = disp.replacesHashId

            disp.dispJson = json.dumps(jsonDict)

            # AFTER the json has been dumped to the disp, convert it for storage
            # in the branch as geom JSON is not stored as a string in the branch
            # Because it's stored in the Disp Tuple/Table "geom" field as a string
            if 'g' in jsonDict:
                jsonDict['g'] = json.loads(jsonDict['g'])

        del newBranchDispItems

    logger.debug("Converted %s disps for %s branches in %s",
                 len(newDisps), len(newBranches),
                 (datetime.now(pytz.utc) - startTime))

    return newDisps, dispIdsToCompile

    # TODO: Something with the LiveDB links


def _convertJsonDispsToTuples(branchTuple: BranchTuple) -> List:
    """ Convert Json Disps to Tuples

     """
    disps: List = []
    for jsonDisp in branchTuple.disps:
        disp = Tuple.smallJsonDictToTuple(jsonDisp)
        disp.coordSetId = branchTuple.coordSetId
        disp.branchId = branchTuple.id
        if hasattr(disp, "geomJson"):
            disp.geomJson = json.dumps(disp.geomJson)
        disps.append(disp)
    return disps



"""Renamed Location indexes

Peek Plugin Database Migration Script

Revision ID: 24b83c5e7e31
Revises: 23117803d414
Create Date: 2017-09-09 23:45:23.530049

"""

# revision identifiers, used by Alembic.
revision = '24b83c5e7e31'
down_revision = '23117803d414'
branch_labels = None
depends_on = None

from alembic import op


def upgrade():
    op.drop_index('idx_DKIndexUpdate_gridKey', table_name='LocationIndexCompiled',
                  schema='pl_diagram')
    op.drop_index('idx_DKIndexUpdate_coordSetId', table_name='LocationIndexCompiled',
                  schema='pl_diagram')
    op.drop_index('idx_DKCompQueue_coordSetId_gridKey',
                  table_name='LocationIndexCompilerQueue', schema='pl_diagram')

    op.create_index('idx_LIIndexUpdate_indexBucket', 'LocationIndexCompiled',
                    ['indexBucket'], unique=True, schema='pl_diagram')
    op.create_index('idx_LIIndexUpdate_modelSetId', 'LocationIndexCompiled',
                    ['modelSetId'], unique=False, schema='pl_diagram')
    op.create_index('idx_LICompQueue_modelSetId_indexBucket',
                    'LocationIndexCompilerQueue', ['modelSetId', 'indexBucket'],
                    unique=False, schema='pl_diagram')
    # ### end Alembic commands ###


def downgrade():
    raise NotImplementedError("Downgrade not implemented")



import logging
import typing
from collections import defaultdict
from datetime import datetime
from typing import List, Dict

import pytz
from peek_plugin_base.worker import CeleryDbConn
from peek_plugin_diagram._private.server.controller.DispCompilerQueueController import \
    DispCompilerQueueController
from peek_plugin_diagram._private.tuples.branch.BranchTuple import BranchTuple
from peek_plugin_base.worker.CeleryApp import celeryApp
from peek_plugin_diagram._private.worker.tasks.ImportDispTask import _bulkInsertDisps
from peek_plugin_diagram._private.worker.tasks.LookupHashConverter import \
    LookupHashConverter
from peek_plugin_diagram._private.worker.tasks._ModelSetUtil import \
    getModelSetIdCoordSetId
from peek_plugin_diagram._private.worker.tasks.branch.BranchDispUpdater import \
    _convertBranchDisps
from peek_plugin_diagram._private.worker.tasks.branch.BranchIndexUpdater import \
    _insertOrUpdateBranches
from peek_plugin_diagram.tuples.branches.ImportBranchTuple import ImportBranchTuple
from txcelery.defer import DeferrableTask
from vortex.Payload import Payload

logger = logging.getLogger(__name__)


@DeferrableTask
@celeryApp.task(bind=True)
def createOrUpdateBranches(self, importBranchesEncodedPayload: bytes) -> None:
    """ Convert Import Branch Tuples

    This method takes import branch tuples, and converts them to
    branch format used throughout the diagram plugin.

    (Thats the packed JSON wrapped by an accessor class)

    """
    # Decode importBranches payload
    importBranches: List[ImportBranchTuple] = (
        Payload().fromEncodedPayload(importBranchesEncodedPayload).tuples
    )

    # Validate the input importBranches
    _validateNewBranchIndexs(importBranches)

    # Do the import
    groupedBranches = _convertImportBranchTuples(importBranches)

    startTime = datetime.now(pytz.utc)

    dbSession = CeleryDbConn.getDbSession()

    engine = CeleryDbConn.getDbEngine()
    conn = engine.connect()
    transaction = conn.begin()

    try:
        for (modelSetKey, modelSetId, coordSetId), branches in groupedBranches.items():
            _insertOrUpdateBranches(conn, modelSetKey, modelSetId, branches)

            newDisps, dispIdsToCompile = _convertBranchDisps(branches)

            # NO TRANSACTION
            # Bulk load the Disps
            _bulkInsertDisps(engine, newDisps)

            # Queue the compiler
            DispCompilerQueueController.queueDispIdsToCompileWithSession(
                dispIdsToCompile, conn
            )

            transaction.commit()
            dbSession.commit()

            logger.debug("Completed importing %s branches for coordSetId %s in %s",
                         len(branches),
                         coordSetId,
                         (datetime.now(pytz.utc) - startTime))

    except Exception as e:
        dbSession.rollback()
        transaction.rollback()
        logger.debug("Retrying createOrUpdateBranches, %s", e)
        logger.exception(e)
        raise self.retry(exc=e, countdown=3)

    finally:
        dbSession.close()
        conn.close()


def _convertImportBranchTuples(importBranches: List[ImportBranchTuple]
                               ) -> Dict[typing.Tuple[str, int, int], List[BranchTuple]]:
    """ Convert Import Branch Tuples

    This method takes import branch tuples, and converts them to
    branch format used throughout the diagram plugin.

    (Thats the packed JSON wrapped by an accessor class)

    """

    # Get a map for the coordSetIds
    modelKeyCoordKeyTuples = [(b.modelSetKey, b.coordSetKey) for b in importBranches]

    coordSetIdByModelKeyCoordKeyTuple = getModelSetIdCoordSetId(modelKeyCoordKeyTuples)

    # Sort out the importBranches by coordSetKey
    branchByModelKeyByCoordKey = defaultdict(lambda: defaultdict(list))
    for importBranch in importBranches:
        branchByModelKeyByCoordKey[importBranch.modelSetKey][importBranch.coordSetKey] \
            .append(importBranch)

    # Define the converted importBranches
    convertedBranchesByCoordSetId: Dict[typing.Tuple[str, int, int], List[BranchTuple]] \
        = {}

    # Get the model set
    dbSession = CeleryDbConn.getDbSession()
    try:
        # Iterate through the importBranches and convert them
        for modelSetKey, item in branchByModelKeyByCoordKey.items():
            for coordSetKey, importBranches in item:
                modelSetId, coordSetId = coordSetIdByModelKeyCoordKeyTuple[
                    (modelSetKey, coordSetKey)]

                lookupHashConverter = LookupHashConverter(
                    dbSession, modelSetId, coordSetId
                )

                convertedBranches = []
                for importBranch in importBranches:
                    branch = BranchTuple.loadFromImportTuple(
                        importBranch, coordSetId,
                        lookupHashConverter=lookupHashConverter
                    )
                    convertedBranches.append(branch)

                convertedBranchesByCoordSetId[(modelSetKey, modelSetId, coordSetId)] \
                    = convertedBranches

    finally:
        dbSession.close()

    return convertedBranchesByCoordSetId


def _validateNewBranchIndexs(newBranches: List[ImportBranchTuple]) -> None:
    for branchIndex in newBranches:
        if not branchIndex.key:
            raise Exception("key is empty for %s" % branchIndex)

        if not branchIndex.modelSetKey:
            raise Exception("modelSetKey is empty for %s" % branchIndex)

        if not branchIndex.coordSetKey:
            raise Exception("coordSetKey is empty for %s" % branchIndex)



"""DispText.textStyleId not null

Peek Plugin Database Migration Script

Revision ID: 1fe753e1f369
Revises: bfdd4f46e293
Create Date: 2017-08-26 17:46:15.455355

"""

# revision identifiers, used by Alembic.
revision = '1fe753e1f369'
down_revision = 'bfdd4f46e293'
branch_labels = None
depends_on = None

import sqlalchemy as sa
from alembic import op


def upgrade():
    op.drop_index('idx_DispText_styleId', table_name='DispText',  schema='pl_diagram')

    op.alter_column('DispText', 'textStyleId', type_=sa.Integer, nullable=False,
                    schema='pl_diagram')
    op.create_index('idx_DispText_styleId', 'DispText', ['textStyleId'], unique=False, schema='pl_diagram')


def downgrade():
    op.alter_column('DispText', 'textStyleId', type_=sa.Integer, nullable=True,
                    schema='pl_diagram')



import logging
from typing import Union

from twisted.internet.defer import Deferred, inlineCallbacks

from peek_plugin_diagram._private.client.controller.BranchIndexCacheController import \
    BranchIndexCacheController
from peek_plugin_diagram._private.tuples.branch.BranchIndexUpdateDateTuple import \
    BranchIndexUpdateDateTuple
from vortex.Payload import Payload
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

logger = logging.getLogger(__name__)


class BranchIndexUpdateDateTupleProvider(TuplesProviderABC):
    def __init__(self, cacheHandler: BranchIndexCacheController):
        self._cacheHandler = cacheHandler

    @inlineCallbacks
    def makeVortexMsg(self, filt: dict,
                      tupleSelector: TupleSelector) -> Union[Deferred, bytes]:
        tuple_ = BranchIndexUpdateDateTuple()
        tuple_.updateDateByChunkKey = {
            key:self._cacheHandler.encodedChunk(key).lastUpdate
            for key in self._cacheHandler.encodedChunkKeys()
        }
        payload = Payload(filt, tuples=[tuple_])
        payloadEnvelope = yield payload.makePayloadEnvelopeDefer()
        vortexMsg = yield payloadEnvelope.toVortexMsg()
        return vortexMsg



from typing import List, Optional
from vortex.Tuple import TupleField, addTupleType, Tuple

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix

@addTupleType
class ImportBranchDeltaColorOverride(Tuple):
    """ Imported Branch Delta Color Override

    This branch delta will change the color of disps.

    """
    __tupleType__ = diagramTuplePrefix + 'ImportBranchDeltaColorOverride'

    #:  A list of keys to apply this color to.
    dispKeys: List[str] = TupleField([])

    #:  The Line Color apples to shape lines
    lineColorHash: Optional[str] = TupleField()

    #:  The Fill Color applies to closed shapes
    fillColorHash: Optional[str] = TupleField()

    #:  This color applies to texts
    colorHash: Optional[str] = TupleField()



import logging
from sqlalchemy import Column, orm, BigInteger
from sqlalchemy import ForeignKey
from sqlalchemy import Integer, String
from sqlalchemy.orm import relationship
from sqlalchemy.sql.schema import Index, Sequence
from vortex.Tuple import Tuple, addTupleType

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix
from peek_plugin_livedb.tuples.LiveDbDisplayValueTuple import LiveDbDisplayValueTuple
from .DeclarativeBase import DeclarativeBase
from .Display import DispBase
from .ModelSet import ModelCoordSet

logger = logging.getLogger(__name__)

LIVE_DB_KEY_DATA_TYPE_BY_DISP_ATTR = {
    'colorId': LiveDbDisplayValueTuple.DATA_TYPE_COLOR,
    'fillColorId': LiveDbDisplayValueTuple.DATA_TYPE_COLOR,
    'lineColorId': LiveDbDisplayValueTuple.DATA_TYPE_COLOR,
    'fillPercent': LiveDbDisplayValueTuple.DATA_TYPE_NUMBER_VALUE,
    'lineStyleId': LiveDbDisplayValueTuple.DATA_TYPE_LINE_STYLE,
    'lineWidth': LiveDbDisplayValueTuple.DATA_TYPE_LINE_WIDTH,
    'text': LiveDbDisplayValueTuple.DATA_TYPE_STRING_VALUE,
    'targetDispGroupId': LiveDbDisplayValueTuple.DATA_TYPE_GROUP_PTR,
}


@addTupleType
class LiveDbDispLink(Tuple, DeclarativeBase):
    __tupleTypeShort__ = 'LDL'
    __tablename__ = 'LiveDbDispLink'
    __tupleType__ = diagramTuplePrefix + __tablename__

    id = Column(BigInteger, primary_key=True, autoincrement=True)

    coordSetId = Column(Integer, ForeignKey('ModelCoordSet.id', ondelete="CASCADE"),
                        nullable=False)
    coordSet = relationship(ModelCoordSet)

    dispId = Column(BigInteger, ForeignKey('DispBase.id', ondelete='CASCADE'),
                    nullable=False)
    disp = relationship(DispBase, backref='liveDbLinks')

    # # comment="The attribute of the disp item to update"
    # dispTableName = Column(String, nullable=False)

    # comment="The attribute of the disp item to update"
    dispAttrName = Column(String, nullable=False)

    liveDbKey = Column(String, nullable=False)

    importKeyHash = Column(String)

    importGroupHash = Column(String)

    importDispHash = Column(String)

    # Store custom props for this link
    propsJson = Column(String)

    __table_args__ = (
        Index("idx_LiveDbDLink_DispKeyHash",
              importKeyHash, importDispHash, dispAttrName, unique=False),
        Index("idx_LiveDbDLink_importGroupHash", importGroupHash, unique=False),
        Index("idx_LiveDbDLink_coordSetId", coordSetId, unique=False),
        Index("idx_LiveDbDLink_dispId", dispId, unique=False),
        Index("idx_LiveDbDLink_dispId_attr", dispId, dispAttrName, unique=True),
        Index("idx_LiveDbDLink_liveKeyId", liveDbKey, unique=False),

        # Designed for faster querying, it only needs to hit the index
        Index("idx_LiveDbDLink_liveDbUpdate", dispId, liveDbKey,
              unique=False),
    )

    # noinspection PyMissingConstructor
    @orm.reconstructor
    def __init__(self, **kwargs):

        for name in self.__fieldNames__:
            if name in kwargs:
                setattr(self, name, kwargs.pop(name))

        assert not kwargs, "LiveDbDispLink.__init__, %s remains" % kwargs



import logging
from typing import Union

from twisted.internet.defer import Deferred, inlineCallbacks

from peek_plugin_diagram._private.server.controller.StatusController import \
    StatusController
from vortex.Payload import Payload
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

logger = logging.getLogger(__name__)


class DiagramLoaderStatusTupleProvider(TuplesProviderABC):
    def __init__(self, statusController: StatusController):
        self._statusController = statusController

    @inlineCallbacks
    def makeVortexMsg(self, filt: dict,
                      tupleSelector: TupleSelector) -> Union[Deferred, bytes]:
        tuples = [self._statusController.status]

        payloadEnvelope = yield Payload(filt, tuples=tuples).makePayloadEnvelopeDefer()
        vortexMsg = yield payloadEnvelope.toVortexMsgDefer()
        return vortexMsg



"""added CoordSet key

Peek Plugin Database Migration Script

Revision ID: bfdd4f46e293
Revises: 2c5f47322bda
Create Date: 2017-08-24 14:08:48.525689

"""

# revision identifiers, used by Alembic.
revision = 'bfdd4f46e293'
down_revision = '2c5f47322bda'
branch_labels = None
depends_on = None

import sqlalchemy as sa
from alembic import op


def upgrade():
    op.add_column('ModelCoordSet', sa.Column('key', sa.String(length=50), nullable=True),
                  schema='pl_diagram')
    op.execute(''' UPDATE "pl_diagram"."ModelCoordSet" SET "key" = "name" ''')
    op.alter_column('ModelCoordSet', 'key', type_=sa.String(length=50), nullable=False,
                    schema='pl_diagram')


def downgrade():
    raise NotImplemented



import logging
from typing import List, Dict

from sqlalchemy import select

from peek_abstract_chunked_index.private.server.client_handlers.ACIChunkUpdateHandlerABC import \
    ACIChunkUpdateHandlerABC
from peek_abstract_chunked_index.private.tuples.ACIEncodedChunkTupleABC import \
    ACIEncodedChunkTupleABC
from peek_plugin_diagram._private.client.controller.LocationIndexCacheController import \
    clientLocationIndexUpdateFromServerFilt
from peek_plugin_diagram._private.storage.LocationIndex import LocationIndexCompiled
from peek_plugin_diagram._private.storage.ModelSet import ModelSet

logger = logging.getLogger(__name__)


class ClientLocationIndexUpdateHandler(ACIChunkUpdateHandlerABC):
    _ChunkedTuple: ACIEncodedChunkTupleABC = LocationIndexCompiled
    _updateFromServerFilt: Dict = clientLocationIndexUpdateFromServerFilt
    _logger: logging.Logger = logger

    @classmethod
    def _makeLoadSql(cls, chunkKeys: List[str]):
        chunkTable = LocationIndexCompiled.__table__
        msTable = ModelSet.__table__

        return select([chunkTable.c.indexBucket,
                       chunkTable.c.blobData,
                       chunkTable.c.lastUpdate,
                       msTable.c.key]) \
            .select_from(chunkTable.join(msTable)) \
            .where(chunkTable.c.indexBucket.in_(chunkKeys))



"""added edgeTemplatesEnabled

Peek Plugin Database Migration Script

Revision ID: 1856b7fb8803
Revises: a86e0317e22a
Create Date: 2019-09-30 16:59:52.193761

"""

# revision identifiers, used by Alembic.
revision = '1856b7fb8803'
down_revision = 'a86e0317e22a'
branch_labels = None
depends_on = None

import sqlalchemy as sa
from alembic import op


def upgrade():
    op.add_column('ModelCoordSet',
                  sa.Column('edgeTemplatesEnabled',
                            sa.Boolean(), nullable=True, server_default='false'),
                  schema='pl_diagram')
    op.execute(''' UPDATE "pl_diagram"."ModelCoordSet"
                   SET "edgeTemplatesEnabled" = false ''')
    op.alter_column('ModelCoordSet', 'edgeTemplatesEnabled', type_=sa.Boolean(),
                    nullable=False, schema='pl_diagram')


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('ModelCoordSet', 'edgeTemplatesEnabled', schema='pl_diagram')
    # ### end Alembic commands ###



"""Added textHeight to DispText

Peek Plugin Database Migration Script

Revision ID: d6e25d8ba156
Revises: 2127d894e46b
Create Date: 2017-08-14 13:27:08.675775

"""

# revision identifiers, used by Alembic.
revision = 'd6e25d8ba156'
down_revision = '2127d894e46b'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():

    op.add_column('DispText',
                  sa.Column('textHeight', sa.Float(), nullable=True),
                  schema='pl_diagram')

    op.add_column('DispText',
                  sa.Column('textHStretch', sa.Float(), nullable=True, server_default='1.0'),
                  schema='pl_diagram')


    op.execute('''
    UPDATE "pl_diagram"."DispText"
    SET "textHStretch" = 1.0
    ''')

    op.alter_column('DispText', 'textHStretch',
                    existing_type=sa.Float(),
                    nullable=False,
                    schema='pl_diagram')


def downgrade():
    raise NotImplementedError("Downgrade is not implemented")


import logging

from peek_abstract_chunked_index.private.server.controller.ACIProcessorStatusControllerABC import \
    ACIProcessorStatusControllerABC
from peek_plugin_diagram._private.tuples.DiagramImporterStatusTuple import \
    DiagramImporterStatusTuple

logger = logging.getLogger(__name__)


class StatusController(ACIProcessorStatusControllerABC):
    NOTIFY_PERIOD = 2.0

    _StateTuple = DiagramImporterStatusTuple



import json
import logging
from collections import defaultdict
from typing import Union, List

from twisted.internet.defer import Deferred

from peek_plugin_diagram._private.client.controller.CoordSetCacheController import \
    CoordSetCacheController
from peek_plugin_diagram._private.client.controller.LocationIndexCacheController import \
    LocationIndexCacheController
from peek_plugin_diagram._private.storage.LocationIndex import LocationIndexCompiled
from peek_plugin_diagram._private.tuples.location_index.DispKeyLocationTuple import \
    DispKeyLocationTuple
from peek_plugin_diagram._private.tuples.location_index.EncodedLocationIndexTuple import \
    EncodedLocationIndexTuple
from peek_plugin_diagram._private.tuples.location_index.LocationIndexTuple import \
    LocationIndexTuple
from peek_plugin_diagram._private.worker.tasks._CalcLocation import dispKeyHashBucket
from vortex.DeferUtil import deferToThreadWrapWithLogger
from vortex.Payload import Payload
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

logger = logging.getLogger(__name__)


class ClientDispKeyLocationTupleProvider(TuplesProviderABC):
    """ Client Location Index Tuple Provider

    This class is used by the UIs when they have the offline location index
    switched off.
    
    """

    def __init__(self, locationCache: LocationIndexCacheController,
                 coordSetCache: CoordSetCacheController):
        self._locationCache = locationCache
        self._coordSetCache = coordSetCache

    @deferToThreadWrapWithLogger(logger)
    def makeVortexMsg(self, filt: dict,
                      tupleSelector: TupleSelector) -> Union[Deferred, bytes]:
        modelSetKey = tupleSelector.selector["modelSetKey"]
        keys = tupleSelector.selector["keys"]

        keysByChunkKey = defaultdict(list)

        foundLocationIndexes: List[LocationIndexTuple] = []

        for key in keys:
            keysByChunkKey[dispKeyHashBucket(modelSetKey, key)].append(key)

        for chunkKey, subKeys in keysByChunkKey.items():
            chunk: EncodedLocationIndexTuple = self._locationCache.encodedChunk(chunkKey)

            if not chunk:
                logger.warning("Location index chunk %s is missing from cache", chunkKey)
                continue

            jsonStr = Payload().fromEncodedPayload(chunk.encodedLocationIndexTuple).tuples[0].jsonStr
            locationsByKey = {i[0]: i[1:] for i in json.loads(jsonStr)}

            for subKey in subKeys:
                if subKey not in locationsByKey:
                    logger.warning(
                        "LocationIndex %s is missing from index, chunkKey %s",
                        subKey, chunkKey
                    )
                    continue

                # Reconstruct the data
                for locationJsonStr in locationsByKey[subKey]:
                    dispLocation = DispKeyLocationTuple.fromLocationJson(locationJsonStr)
                    foundLocationIndexes.append(dispLocation)

                    # Populate the coord set key
                    coordSet = self._coordSetCache.coordSetForId(dispLocation.coordSetId)

                    if coordSet is None:
                        logger.warning("Can not find coordSet with ID %s",
                                       dispLocation.coordSetId)
                        continue

                    dispLocation.coordSetKey = coordSet.key

        # Create the vortex message
        return Payload(filt,
                       tuples=foundLocationIndexes).makePayloadEnvelope().toVortexMsg()



import json
import logging
from collections import defaultdict
from typing import Union, List

from twisted.internet.defer import Deferred
from vortex.DeferUtil import deferToThreadWrapWithLogger
from vortex.Payload import Payload
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

from peek_plugin_diagram._private.client.controller.BranchIndexCacheController import \
    BranchIndexCacheController
from peek_plugin_diagram._private.storage.branch.BranchIndexEncodedChunk import \
    BranchIndexEncodedChunk
from peek_plugin_diagram._private.worker.tasks.branch._BranchIndexCalcChunkKey import \
    makeChunkKeyForBranchIndex
from peek_plugin_diagram._private.tuples.branch.BranchTuple import BranchTuple

logger = logging.getLogger(__name__)


class BranchTupleProvider(TuplesProviderABC):
    def __init__(self, cacheHandler: BranchIndexCacheController):
        self._cacheHandler = cacheHandler

    @deferToThreadWrapWithLogger(logger)
    def makeVortexMsg(self, filt: dict,
                      tupleSelector: TupleSelector) -> Union[Deferred, bytes]:
        modelSetKey = tupleSelector.selector["modelSetKey"]
        coordSetId = tupleSelector.selector["coordSetId"]
        keys = tupleSelector.selector["keys"]

        keysByChunkKey = defaultdict(list)

        results: List[BranchTuple] = []

        for key in keys:
            keysByChunkKey[makeChunkKeyForBranchIndex(modelSetKey, key)].append(key)

        for chunkKey, branchKeys in keysByChunkKey.items():
            chunk: BranchIndexEncodedChunk = self._cacheHandler.encodedChunk(chunkKey)

            if not chunk:
                logger.warning("BranchIndex chunk %s is missing from cache", chunkKey)
                continue

            resultsByKeyStr = Payload().fromEncodedPayload(chunk.encodedData).tuples[0]
            resultsByKey = json.loads(resultsByKeyStr)

            for branchKey in branchKeys:
                if branchKey not in resultsByKey:
                    logger.warning(
                        "Branch %s is missing from index, chunkKey %s",
                        branchKey, chunkKey
                    )
                    continue

                packedJsons = resultsByKey[branchKey]

                for packedJson  in packedJsons:
                    result = BranchTuple.loadFromJson(packedJson, None, None)
                    if result.coordSetId == coordSetId or coordSetId is None:
                        results.append(result)

        # Create the vortex message
        return Payload(filt, tuples=results).makePayloadEnvelope().toVortexMsg()



from datetime import datetime

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix
from peek_plugin_diagram._private.tuples.branch.BranchLiveEditTupleAction import \
    BranchLiveEditTupleAction
from peek_plugin_diagram._private.tuples.branch.BranchTuple import BranchTuple
from vortex.Tuple import Tuple, addTupleType, TupleField


@addTupleType
class BranchLiveEditTuple(Tuple):
    """ Branch Live Edit Tuple

    This tuple is used internally to transfer branches between UIs that are actively
    editing the.

    This isn't stored anywhere, it just gets relayed between multiple UIs.

    """
    __tupleType__ = diagramTuplePrefix + 'BranchLiveEditTuple'

    EDITING_STARTED = BranchLiveEditTupleAction.EDITING_STARTED
    EDITING_UPDATED = BranchLiveEditTupleAction.EDITING_UPDATED
    EDITING_FINISHED = BranchLiveEditTupleAction.EDITING_FINISHED
    EDITING_SAVED = BranchLiveEditTupleAction.EDITING_SAVED

    # This field is server side only
    branchTuple: BranchTuple = TupleField()
    updatedByUser: str = TupleField()

    uiUpdateDate: datetime = TupleField()
    serverUpdateDate: datetime = TupleField()

    updateFromActionType: int = TupleField()


from vortex.Tuple import addTupleType, TupleField
from vortex.TupleAction import TupleActionABC

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix
from peek_plugin_diagram._private.tuples.branch.BranchTuple import BranchTuple


@addTupleType
class BranchUpdateTupleAction(TupleActionABC):
    """ Branch Update Tuple Action

    This is the Branch Update tuple Action

    """
    __tupleType__ = diagramTuplePrefix + 'BranchUpdateTupleAction'

    doDelete: bool = TupleField(False)
    modelSetId:int = TupleField()
    branchTuple: BranchTuple = TupleField()



"""moved smallest sizes to grids

Peek Plugin Database Migration Script

Revision ID: 2127d894e46b
Revises: 6a97dd344aed
Create Date: 2017-08-13 21:31:51.165359

"""

# revision identifiers, used by Alembic.
revision = '2127d894e46b'
down_revision = '6a97dd344aed'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2
from sqlalchemy.dialects import postgresql

def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('ModelCoordSet','smallestTextSize', schema='pl_diagram')

    op.drop_column('ModelCoordSet','smallestShapeSize', schema='pl_diagram')

    op.add_column('ModelCoordSetGridSize',
                  sa.Column('smallestTextSize', sa.Float(), nullable=True, server_default='6.0'),
                  schema='pl_diagram')

    op.add_column('ModelCoordSetGridSize',
                  sa.Column('smallestShapeSize', sa.Float(), nullable=True, server_default='2.0'),
                  schema='pl_diagram')


    op.execute('''
    UPDATE "pl_diagram"."ModelCoordSetGridSize"
    SET "smallestShapeSize" = 2.0,
        "smallestTextSize" = 6.0
    ''')

    op.alter_column('ModelCoordSetGridSize', 'smallestTextSize',
                    existing_type=sa.Float(),
                    nullable=False,
                    schema='pl_diagram')

    op.alter_column('ModelCoordSetGridSize', 'smallestShapeSize',
                    existing_type=sa.Float(),
                    nullable=False,
                    schema='pl_diagram')


def downgrade():
    raise NotImplementedError("Downgrade is not implemented")